import parseurl from 'parseurl';
import { HttpError } from 'http-errors';
import finalhandler from 'finalhandler';
import { mergeManifests } from '@open-xchange/vite-plugin-ox-manifests';
import logger from '../utils/logger.js';
import { joinUrlPaths, getStack } from '../utils/util.js';
import * as proxy from '../http/proxy.js';
export default function manifestMiddleware(frontends, base) {
    const manifestsPath = joinUrlPaths(base, '/manifests');
    return async function (req, res, next) {
        if (req.headers['proxied-by-ox-proxy'])
            return next();
        const path = parseurl(req)?.path;
        if (path !== manifestsPath)
            return next();
        try {
            logger.info(`Fetch manifests from "${frontends.map(f => f.url).join(', ')}"`);
            const responses = await proxy.all(req, res, frontends.map(({ url, secure }) => ({ url, secure, headers: { 'proxied-by-ox-proxy': 'true' } })));
            if (!responses)
                throw new Error();
            const data = mergeManifests(responses.flat().reverse());
            logger.info('Merged manifests');
            // merge all manifests
            res.statusCode = 200;
            res.setHeader('Content-Type', 'application/json');
            res.write(JSON.stringify(data));
            return res.end();
        }
        catch (err) {
            const handle = finalhandler(req, res);
            if (err instanceof HttpError) {
                // in case of http-error the proxy explicitly received an error code
                logger.error(`Could not fetch manifests: ${getStack(err)}`);
                return handle(err);
            }
            logger.error(`Error while fetching manifests: ${getStack(err)}`);
            return handle();
        }
    };
}
