import parseurl from 'parseurl';
import { getStack, shouldHandle } from '../utils/util.js';
import logger from '../utils/logger.js';
import { createPath, createHeaders } from '../http/http.js';
import * as proxy from '../http/proxy.js';
export default function upgradeMiddleware(wsProxies) {
    return async function (req, socket, head) {
        socket.on('error', err => logger.error(`Socket error on upgrade: ${getStack(err)}`));
        const path = parseurl(req)?.path;
        if (!path)
            return;
        for (const wsProxy of wsProxies) {
            if (shouldHandle(wsProxy.condition, path)) {
                try {
                    const { url: target, secure } = wsProxy;
                    logger.info(`Websocket at "${path}" is handled by vite-plugin-proxy`);
                    // http2-proxy does a changeOrigin as a default
                    const headers = createHeaders(req, wsProxy);
                    const newPath = createPath(path, wsProxy);
                    return await proxy.ws(req, socket, head, { url: target, secure, headers, path: newPath });
                }
                catch (err) {
                    logger.error(`Upgrade of websocket at "${path}" failed: ${getStack(err)}`);
                    return;
                }
            }
        }
    };
}
